﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace TestDameKnihu {
	class Utils {
		/// <summary>
		/// Serializes the object as JSON
		/// </summary>
		/// <param name="obj"></param>
		/// <returns></returns>
		public static string SerializeToJSON(object obj, bool format = false, string valueIfCyclic = null) {
			if (obj == null) {
				return null;
			}
			if (valueIfCyclic == null) {
				return Newtonsoft.Json.JsonConvert.SerializeObject(
					obj,
					new Newtonsoft.Json.JsonSerializerSettings {
						Formatting = format ? Newtonsoft.Json.Formatting.Indented : Newtonsoft.Json.Formatting.None,
						NullValueHandling = Newtonsoft.Json.NullValueHandling.Ignore
					}
				);
			} else {
				try {
					return Newtonsoft.Json.JsonConvert.SerializeObject(
						obj,
						new Newtonsoft.Json.JsonSerializerSettings {
							Formatting = format ? Newtonsoft.Json.Formatting.Indented : Newtonsoft.Json.Formatting.None,
							NullValueHandling = Newtonsoft.Json.NullValueHandling.Ignore
						}
					);
				} catch {
					return valueIfCyclic;
				}
			}
		}

		public static (System.Net.HttpStatusCode, byte[]) PostFormToURLRaw(string url, string contentType, string data, IDictionary<string, string> headers = null) {
			byte[] byteArray = Encoding.ASCII.GetBytes(data);
			return PostToURLRaw(url, contentType, byteArray, headers);
		}

		public static (System.Net.HttpStatusCode, byte[]) PostToURLRaw(string url, string contentType, byte[] data, IDictionary<string, string> headers = null) {
			var webClientFetchURLandDo = new System.Net.WebClient();
			webClientFetchURLandDo.Encoding = GetEncoding(contentType, System.Text.Encoding.UTF8);
			webClientFetchURLandDo.Headers.Set("Content-type", contentType);
			if (headers != null) {
				foreach (var header in headers) {
					webClientFetchURLandDo.Headers.Set(header.Key, header.Value);
				}
			}
			try {
				return (System.Net.HttpStatusCode.OK, webClientFetchURLandDo.UploadData(new Uri(url), "POST", data));
			} catch (System.Net.WebException ex) {
				if (ex.Response as System.Net.HttpWebResponse != null) {
					var stream = ex.Response.GetResponseStream();
					MemoryStream ms = new MemoryStream();
					stream.CopyTo(ms);
					return ((ex.Response as System.Net.HttpWebResponse).StatusCode, ms.ToArray());
				} else { throw; }
			}
		}

		private static Encoding GetEncoding(string contentType, Encoding def) {
			try {
				if (System.Net.Http.Headers.NameValueWithParametersHeaderValue.TryParse(contentType.Replace('/', '_'), out var hdr)) {// the imbeciles throw an exception if there's a slash!!!
					var charset = hdr.Parameters.Where(p => p.Name.Equals("charset", StringComparison.OrdinalIgnoreCase)).Select(p => p.Value).FirstOrDefault();
					return string.IsNullOrEmpty(charset) ? def : Encoding.GetEncoding(charset);
				}
			} catch { }
			return def;
		}
	}
}
